package cn.com.qmth.examcloud.ws.api.provider;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import cn.com.qmth.examcloud.ws.api.WsCloudService;
import cn.com.qmth.examcloud.ws.api.request.SendTextReq;
import cn.com.qmth.examcloud.ws.api.response.SendTextResp;
import cn.com.qmth.examcloud.ws.starter.core.MessageOut;
import cn.com.qmth.examcloud.ws.starter.core.SessionInfo;
import cn.com.qmth.examcloud.ws.starter.core.WebSocketHelper;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.ThreadContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.websocket.Session;

@Api(tags = "WebSocket服务")
@RestController
@RequestMapping("api/cloud/ws")
public class WsCloudServiceProvider implements WsCloudService {

    private static final long serialVersionUID = 4784116669909937047L;

    private static final Logger log = LoggerFactory.getLogger(WsCloudServiceProvider.class);

    @ApiOperation(value = "RPC发送消息接口")
    @PostMapping("sendText")
    @Override
    public SendTextResp sendText(@RequestBody SendTextReq req) {
        Long rootOrgId = req.getRootOrgId();
        UserType userType = req.getUserType();
        Long userId = req.getUserId();
        String path = req.getPath();
        String eventId = req.getEventId();
        String content = req.getContent();

        User user = new User();
        user.setUserType(userType);
        user.setUserId(userId);
        user.setRootOrgId(rootOrgId);
        String userKey = user.buildKey();

        Session session = WebSocketHelper.getSession(path, userKey);
        if (null == session) {
            log.error("NO_WS_SESSION user={} path={}", userKey, path);
            throw new StatusException("100001", "WS会话不存在");
        }

        SessionInfo sessionInfo = WebSocketHelper.getSessionInfo(session);
        if (null == sessionInfo) {
            log.error("NO_SESSION_INFO user={} path={}", userKey, path);
            throw new StatusException("100002", "WS信息不存在");
        }

        MessageOut out = new MessageOut(path, sessionInfo.getSessionId());
        out.setEventId(eventId);

        JsonElement jsonEl = null;
        if (StringUtils.isNotBlank(content)) {
            try {
                jsonEl = JsonParser.parseString(content);
            } catch (JsonSyntaxException e) {
                log.error("WS_MESSAGE_INVALID user={} path={}", userKey, path);
                throw new StatusException("100003", "WS信息格式有误");
            }
        }

        out.setContent(jsonEl);

        try {
            ThreadContext.put("TRACE_ID", sessionInfo.getSessionId());
            WebSocketHelper.sendText(session, path, out);
        } finally {
            ThreadContext.put("TRACE_ID", ThreadLocalUtil.getTraceId());
        }

        SendTextResp resp = new SendTextResp();
        resp.setSessionId(sessionInfo.getSessionId());
        return resp;
    }

}
